#include "avr/3310_routines.h"
#include "avr/3310_routines.c"
#include "avr/io.h"
#include "stdlib.h"
#include "stdio.h"
#include "avr/interrupt.h"
#include "avr/eeprom.h"
#include "avr/wdt.h"

#define uninited 0	/* display mode */
#define startup 1	/* display mode */
#define normal 2	/* display mode */
#define menu 3		/* display mode */
#define escape 66	/* button constant, arbitrary value */
#define enter 67	/* button constant, arbitrary value */
#define up 68		/* button constant, arbitrary value */
#define down 69		/* button constant, arbitrary value */
#define bktreg OCR0	/* PWM register for backlight */
#define enter_pin (PINE & 0b10000000)
#define escape_pin (PINE & 0b01000000)
#define up_pin (PINE & 0b00100000)
#define down_pin (PINE & 0b00010000)

#define panel_pin (PIND & 0b00010000)
#define maxitems 5
#define itemlength 12

#define bktoffaddr 11			/* EEPROM address for bktofflevel */
#define bktonaddr 12			/* EEPROM address for bktonlevel */
#define contrastaddr 13			/* EEPROM address for contrast */
#define item1posaddr 14			/* EEPROM address for item1pos */
#define item2posaddr 15			/* EEPROM address for item2pos */
#define item3posaddr 16			/* EEPROM address for item3pos */
#define item4posaddr 17			/* EEPROM address for item4pos */
#define item5posaddr 18			/* EEPROM address for item5pos */
#define dispoffnegaddr 19			/* EEPROM address for dispoffneg */
#define disponnegaddr 20			/* EEPROM address for disponneg */


#define escape_vect PCINT1_vect	/* interrupt vector for button escape */
#define up_vect INT0_vect		/* interrupt vector for button up */
#define down_vect INT1_vect		/* interrupt vector for button down */

unsigned char menuselect = 1;	/* global variable for menu selection */
unsigned char sercount = 7;		/* global variable for interrupt2 scale */
unsigned char dispmode = normal;/* global variable for LCD mode, can be menu, normal, uninited, startup*/
unsigned char menulevel = 0;	/* global variable for menu item selection */
unsigned char bktofflevel = 255;/* global variable for backlight level when dashlight is off */
unsigned char bktonlevel = 255;	/* global variable for backlight level when dashlight is on */
unsigned char contrast = 0xC8;	/* global variable for LCD contrast, 0xc8 is safe default */
unsigned char posy = 0;			/* pointer for info items */
unsigned char posx = 0;			/* pointer for info items */
unsigned char item1pos = 0;
unsigned char item2pos = 0;
unsigned char item3pos = 0;
unsigned char item4pos = 0;
unsigned char item5pos = 0;
unsigned char disponneg = 0;
unsigned char dispoffneg = 1;
unsigned char updateneeded = 1;	/* tells if LCD needs updating */

char item[maxitems][itemlength];					/* global string for LCD items */


void lcd_string(char *str) 		/* this function writes a string to LCD */
{
	int i;
	for(i = 0; str[i] != 0; i++) {
		LCD_writeChar(str[i]);

	}
}

void setup(void){				/* hardware and global vars setup */
	DDRA = 0b00000000;
	DDRB = 0b11110111;
	DDRC = 0b00000000;
	DDRD = 0b00001000;
	DDRE = 0b00000010;
	DDRF = 0b00000000;
	

	PORTB = 0b11000000;
	PORTD = 0b10011101;
	PORTC = 0b11111111;
	PORTE = 0b11110000;		/*Pullups for buttons*/

	TCCR0 = 0b01111011; /* fast PWM, set at TOP, clear on match */
	//TCCR0B = 0b00000001; /* prescaler at 0 */
	TCCR2 = 0b00000101; /* Prescaler at 1024 */
	TIMSK = 0b01000000; /* tim2 ovf interrupt enable, 14Hz. */
	UCSR0A = 0b00000000;
	UCSR0B = 0b10011000; /* Receiver transmitter enabled, RXC interrupt enabled */
	UCSR0C = 0b00000110; /* 8 bits */
	UBRR0L = 23;			 /* 19k2 at 3686400Hz */
	//WDTCR = 0b00001111;		/* Watchdog timer at 1,1s nominal) */
	//EICRA = 0b00001010;	 /* Falling edge interrupt on int0+1 */
	//EIMSK = 0b00000011;	 /* Int0+1 enabled */
	//PCICR = 0b00000110;  /* Pin change interrupts 1+2 enabled */
	//PCMSK2 = 0b00010000; /* PCINT20 enabled */
	//PCMSK1 = 0b00000001; /* PCINT8 enabled */
	
	//sprintf(item[3], "teststring");
	
	if((eeprom_read_byte((uint8_t*)bktoffaddr)) == 0xff){		/*Checks for and loads default values into*/
		eeprom_write_byte((uint8_t*)bktoffaddr, 0x00);			/*an empty EEPROM*/
		}
	if((eeprom_read_byte((uint8_t*)contrastaddr)) == 0xff){
		eeprom_write_byte((uint8_t*)contrastaddr, 0xc8);
		}	
	if((eeprom_read_byte((uint8_t*)disponnegaddr)) == 0xff){
		eeprom_write_byte((uint8_t*)disponnegaddr, 0x00);
		}	
	if((eeprom_read_byte((uint8_t*)dispoffnegaddr)) == 0xff){
		eeprom_write_byte((uint8_t*)dispoffnegaddr, 0x01);
		}	
	bktofflevel = eeprom_read_byte((uint8_t*)bktoffaddr);	/* Loads backlight level when panel off from EEPROM */
	bktonlevel = eeprom_read_byte((uint8_t*)bktonaddr);		/* Loads backlight level when panel on from EEPROM */
	contrast = eeprom_read_byte((uint8_t*)contrastaddr);	/* Loads display contrast from EEPROM */
	item1pos = eeprom_read_byte((uint8_t*)item1posaddr);	/* Loads item1pos from EEPROM */
	item2pos = eeprom_read_byte((uint8_t*)item2posaddr);	/* Loads item1pos from EEPROM */
	item3pos = eeprom_read_byte((uint8_t*)item3posaddr);	/* Loads item1pos from EEPROM */
	item4pos = eeprom_read_byte((uint8_t*)item4posaddr);	/* Loads item1pos from EEPROM */
	item5pos = eeprom_read_byte((uint8_t*)item5posaddr);	/* Loads item1pos from EEPROM */
	dispoffneg = eeprom_read_byte((uint8_t*)dispoffnegaddr);	/* Loads dispoffneg from EEPROM */
	disponneg = eeprom_read_byte((uint8_t*)disponnegaddr);	/* Loads disponn from EEPROM */

	spi_init();			 /* initializes SPI, function located in 3310_routines.c */
	//LCD_init(contrast, 1);		 /* initializes the LCD display, function located in 3310_routines.c */
	}



void LCD_update(void)	/* handles all info on the LCD display including backlight*/
{
char str[12];


	/*if(!panel_pin){		 panel light is on 
		bktreg = bktonlevel;
		LCD_init(contrast, disponneg);	
	}
	if(panel_pin){
		bktreg = bktofflevel;
		LCD_init(contrast, dispoffneg);
	} */
	
	if(dispmode == uninited){	/* power ON screen */
		LCD_clear();
		LCD_gotoXY(0,1);
		sprintf(str, "ON");
		lcd_string(str);	
	}
	if(dispmode == startup){	/* startup screen */
		LCD_clear();
		LCD_gotoXY(3,0);
		sprintf(str, "Autotronics");
		lcd_string(str);
		LCD_gotoXY(16,2);
		sprintf(str, "KE-Flex");
		lcd_string(str);
		LCD_gotoXY(12,3);
		sprintf(str, "V0.1beta");
		lcd_string(str);
	}	
	if(dispmode == normal){		/* normal screen, selected information appears */
		LCD_clear();
		LCD_gotoXY(0,0);
		LCD_writepat(0b00101000, 21);
		sprintf(str, "Status");
		lcd_string(str);
		LCD_writepat(0b00101000, 21);
		LCD_gotoXY(0,1);
		if(item1pos != 0xff) lcd_string(item[item1pos]);
		LCD_gotoXY(0,2);
		if(item2pos != 0xff) lcd_string(item[item2pos]);
		LCD_gotoXY(0,3);
		if(item3pos != 0xff) lcd_string(item[item3pos]);
		LCD_gotoXY(0,4);
		if(item4pos != 0xff) lcd_string(item[item4pos]);
		LCD_gotoXY(0,5);	
		if(item5pos != 0xff) lcd_string(item[item5pos]);	
	}
	if(dispmode == menu){		/* display is in menu mode */

		if(menulevel == 0){
			LCD_clear();
			LCD_gotoXY(0,0);
			LCD_writepat(0b00101000, 28);
			sprintf(str, "Menu");
			lcd_string(str);
			LCD_writepat(0b00101000, 28);
			LCD_gotoXY(14,1);
			sprintf(str, "Datap  10");
			lcd_string(str);
			LCD_gotoXY(14,2);
			sprintf(str, "Setup  20");
			lcd_string(str);
			LCD_gotoXY(14,3);
			sprintf(str, "Stats  30");
			lcd_string(str);
			LCD_gotoXY(14,4);
			sprintf(str, "About  40");
			lcd_string(str);
			LCD_gotoXY(0,menuselect);
			LCD_writeChar('>');
		}
		if(menulevel == 1){	/* menu 10 */
			LCD_clear();
			LCD_gotoXY(0,0);
			LCD_writepat(0b00101000, 7);
			sprintf(str, "Info items");
			lcd_string(str);
			LCD_writepat(0b00101000, 7);
			LCD_gotoXY(14,1);
			sprintf(str, "Item1   11");
			lcd_string(str);
			LCD_gotoXY(14,2);
			sprintf(str, "Item2   12");
			lcd_string(str);
			LCD_gotoXY(14,3);
			sprintf(str, "Item3   13");
			lcd_string(str);
			LCD_gotoXY(14,4);
			sprintf(str, "Item4   14");
			lcd_string(str);
			LCD_gotoXY(14,5);
			sprintf(str, "Item5   15");
			lcd_string(str);
						
			LCD_gotoXY(0,menuselect);
			LCD_writeChar('>');
		}
		if(menulevel == 2){	
			LCD_clear();
			LCD_gotoXY(0,0);
			LCD_writepat(0b00101000, 7);
			sprintf(str, "Disp setup");
			lcd_string(str);
			LCD_writepat(0b00101000, 7);
			LCD_gotoXY(14,1);
			sprintf(str, "Backlt off");
			lcd_string(str);
			LCD_gotoXY(14,2);
			sprintf(str, "Backlt on");
			lcd_string(str);
			LCD_gotoXY(14,3);
			sprintf(str, "Contrast");
			lcd_string(str);
			LCD_gotoXY(14,4);
			sprintf(str, "Dispng off"); /* level 24 */
			lcd_string(str);
			LCD_gotoXY(14,5);
			sprintf(str, "Dispng on"); /* level 25 */
			lcd_string(str); 
						
			LCD_gotoXY(0,menuselect);
			LCD_writeChar('>');	
		}
		if(menulevel == 3){	
			LCD_clear();
			LCD_gotoXY(0,0);
			LCD_writepat(0b00101000, 7);
			sprintf(str, "Statistics");
			lcd_string(str);
			LCD_writepat(0b00101000, 7);
			LCD_gotoXY(14,1);
			sprintf(str, "Fuelpr. 31");
			lcd_string(str);
			LCD_gotoXY(14,2);
			sprintf(str, "Cldstrt 32");
			lcd_string(str);
		/*	LCD_gotoXY(14,3);
			sprintf(str, "        33");
			lcd_string(str);
			LCD_gotoXY(14,4);
			sprintf(str, "        34");
			lcd_string(str);
			LCD_gotoXY(14,5);
			sprintf(str, "        35");
			lcd_string(str); */
						
			LCD_gotoXY(0,menuselect);
			LCD_writeChar('>');	
		}
		if(menulevel == 4){
			LCD_clear();
			LCD_gotoXY(0,0);
			//LCD_writelogo(0);
			LCD_gotoXY(0,1);
			sprintf(str, "Nothing");
			lcd_string(str);	
			LCD_gotoXY(0,2);
			sprintf(str, "to see");
			lcd_string(str);
			LCD_gotoXY(0,3);
			sprintf(str, "here");
			lcd_string(str);
		}
		if(menulevel == 11){	
			LCD_clear();
			LCD_gotoXY(0,0);
			LCD_writepat(0b00101000, 7);
			sprintf(str, "Infoitem 1");
			lcd_string(str);
			LCD_writepat(0b00101000, 7);
			LCD_gotoXY(0,3);
			lcd_string(item[item1pos]);
			LCD_gotoXY(0,4);
			sprintf(str, "(Item %d)", item1pos);
			lcd_string(str);
		}
		if(menulevel == 12){	
			LCD_clear();
			LCD_gotoXY(0,0);
			LCD_writepat(0b00101000, 7);
			sprintf(str, "Infoitem 2");
			lcd_string(str);
			LCD_writepat(0b00101000, 7);
			LCD_gotoXY(0,3);
			lcd_string(item[item2pos]);
			LCD_gotoXY(0,4);
			sprintf(str, "(Item %d)", item2pos);
			lcd_string(str);
		}
		if(menulevel == 13){	
			LCD_clear();
			LCD_gotoXY(0,0);
			LCD_writepat(0b00101000, 7);
			sprintf(str, "Infoitem 3");
			lcd_string(str);
			LCD_writepat(0b00101000, 7);
			LCD_gotoXY(0,3);
			lcd_string(item[item3pos]);
			LCD_gotoXY(0,4);
			sprintf(str, "(Item %d)", item3pos);
			lcd_string(str);
		}
		if(menulevel == 14){	
			LCD_clear();
			LCD_gotoXY(0,0);
			LCD_writepat(0b00101000, 7);
			sprintf(str, "Infoitem 4");
			lcd_string(str);
			LCD_writepat(0b00101000, 7);
			LCD_gotoXY(0,3);
			lcd_string(item[item4pos]);
			LCD_gotoXY(0,4);
			sprintf(str, "(Item %d)", item4pos);
			lcd_string(str);
		}
		if(menulevel == 15){	
			LCD_clear();
			LCD_gotoXY(0,0);
			LCD_writepat(0b00101000, 7);
			sprintf(str, "Infoitem 5");
			lcd_string(str);
			LCD_writepat(0b00101000, 7);
			LCD_gotoXY(0,3);
			lcd_string(item[item5pos]);
			LCD_gotoXY(0,4);
			sprintf(str, "(Item %d)", item5pos);
			lcd_string(str);
		}
		if(menulevel == 21){	
			LCD_clear();
			LCD_gotoXY(0,0);
			LCD_writepat(0b00101000, 11);
			sprintf(str, "Backlight");
			lcd_string(str);
			LCD_writepat(0b00101000, 10);
			LCD_gotoXY(11,1);
			sprintf(str, "Panel off");
			lcd_string(str);
			LCD_gotoXY(10,2);
			LCD_writepat(0b01111111, (255 - bktofflevel)/4);
			LCD_gotoXY(33,3);
			sprintf(str, "%d", (255 - bktofflevel)/5);
			lcd_string(str);
		}
		if(menulevel == 22){	
			LCD_clear();
			LCD_gotoXY(0,0);
			LCD_writepat(0b00101000, 11);
			sprintf(str, "Backlight");
			lcd_string(str);
			LCD_writepat(0b00101000, 10);
			LCD_gotoXY(11,1);
			sprintf(str, "Panel on");
			lcd_string(str);
			LCD_gotoXY(10,2);
			LCD_writepat(0b01111111, ((255 - bktonlevel)/4));
			LCD_gotoXY(33,3);
			sprintf(str, "%d", (255 - bktonlevel)/5);
			lcd_string(str);
		}
		if(menulevel == 23){	
			LCD_clear();
			LCD_gotoXY(0,0);
			LCD_writepat(0b00101000, 14);
			sprintf(str, "Contrast");
			lcd_string(str);
			LCD_writepat(0b00101000, 14);
			LCD_gotoXY(30,2);
			LCD_writepat(0b01111111, (contrast-0xbb));
			LCD_gotoXY(33,3);
			sprintf(str, "%d", (contrast-0xbb));
			lcd_string(str);
		}
		if(menulevel == 24){	
			LCD_clear();
			LCD_gotoXY(0,0);
			LCD_writepat(0b00101000, 3);
			sprintf(str, "Disp invert");
			lcd_string(str);
			LCD_writepat(0b00101000, 4);
			LCD_gotoXY(0,1);
			sprintf(str, "if panel off");
			lcd_string(str);
			LCD_gotoXY(3,3);
			if(dispoffneg){
				sprintf(str, "Yes");
			}
			if(!dispoffneg){
				sprintf(str, "No");
			}
			lcd_string(str);
		}
		if(menulevel == 25){	
			LCD_clear();
			LCD_gotoXY(0,0);
			LCD_writepat(0b00101000, 3);
			sprintf(str, "Disp invert");
			lcd_string(str);
			LCD_writepat(0b00101000, 4);
			LCD_gotoXY(3,1);
			sprintf(str, "if panel on");
			lcd_string(str);
			LCD_gotoXY(3,3);
			if(disponneg){
				sprintf(str, "Yes");
			}
			if(!disponneg){
				sprintf(str, "No");
			}
			lcd_string(str);
		}
	}
/*	LCD_gotoXY(0,0);
	sprintf(str, "%d %d %d", dispmode, menulevel, menuselect);
	lcd_string(str);*/
	
}

void buttonmanager(unsigned char button) /* manages all button presses */
{

	if(button == escape){
		if(menulevel == 0){
			dispmode = normal;
		}
		if((menulevel == 21) && (bktofflevel != eeprom_read_byte((uint8_t*)bktoffaddr)));{
			bktofflevel = eeprom_read_byte((uint8_t*)bktoffaddr);
		}
		if((menulevel == 22) && (bktonlevel != eeprom_read_byte((uint8_t*)bktonaddr)));{
			bktonlevel = eeprom_read_byte((uint8_t*)bktonaddr);
		}		
		if((menulevel == 23) && (contrast != eeprom_read_byte((uint8_t*)contrastaddr)));{
			contrast = eeprom_read_byte((uint8_t*)contrastaddr);
		}
		menuselect = menulevel % 10;
		menulevel /= 10;
		return;
	}	

	if((button == up) && (menulevel < 10)){
		menuselect--;
		if(menuselect == 0){
			menuselect = 1;
		}
		return;
	}
	
	if(menulevel == 11){
		if((button == down) && (item1pos < maxitems)){
			item1pos++;
		}
		if((button == up) && (item1pos > 0)){
			item1pos--;
		}
		if(button == enter){
			menuselect = menulevel % 10;
			menulevel /= 10;
;			eeprom_write_byte((uint8_t*)item1posaddr, item1pos);
		}
		return;
	}
	if(menulevel == 12){
		if((button == down) && (item2pos < maxitems)){
			item2pos++;
		}
		if((button == up) && (item2pos > 0)){
			item2pos--;
		}
		if(button == enter){
			menuselect = menulevel % 10;
			menulevel /= 10;
;			eeprom_write_byte((uint8_t*)item2posaddr, item2pos);
		}
		return;
	}
	if(menulevel == 13){
		if((button == down) && (item3pos < maxitems)){
			item3pos++;
		}
		if((button == up) && (item3pos > 0)){
			item3pos--;
		}
		if(button == enter){
			menuselect = menulevel % 10;
			menulevel /= 10;
			eeprom_write_byte((uint8_t*)item3posaddr, item3pos);
		}
		return;
	}
	if(menulevel == 14){
		if((button == down) && (item1pos < maxitems)){
			item4pos++;
		}
		if((button == up) && (item4pos > 0)){
			item4pos--;
		}
		if(button == enter){
			menuselect = menulevel % 10;
			menulevel /= 10;
			eeprom_write_byte((uint8_t*)item4posaddr, item4pos);
		}
		return;
	}
	if(menulevel == 15){
		if((button == down) && (item5pos < maxitems)){
			item5pos++;
		}
		if((button == up) && (item5pos > 0)){
			item5pos--;
		}
		if(button == enter){
			menuselect = menulevel % 10;
			menulevel /= 10;
			eeprom_write_byte((uint8_t*)item5posaddr, item5pos);
		}
		return;
	}	
	if(menulevel == 21){
		if(button == down && bktofflevel < 255){
			bktofflevel += 5;
		}
		if(button == up && bktofflevel > 0){
			bktofflevel -= 5;
		}
		if(button == enter){
			menuselect = menulevel % 10;
			menulevel /= 10;
			eeprom_write_byte((uint8_t*)bktoffaddr, bktofflevel);
		}
		bktreg = bktofflevel;
		return;
	}
	
	if(menulevel == 22){
		if(button == down && bktonlevel < 255){
			bktonlevel += 5;
		}
		if(button == up && bktonlevel > 0){
			bktonlevel -= 5;
		}
		if(button == enter){
			menuselect = menulevel % 10;
			menulevel /= 10;
			eeprom_write_byte((uint8_t*)bktonaddr, bktonlevel);
		}
		bktreg = bktonlevel;
		return;
	}
	
	if(menulevel == 23){
		if((button == up) && (contrast < 0xDD)){
			contrast++;
		}
		if ((button == down) && (contrast > 0xBB)){
			contrast--;
		}
		if(button == enter){
			menuselect = menulevel % 10;
			menulevel /= 10;
			eeprom_write_byte((uint8_t*)contrastaddr, contrast);
		}
		return;
	}
	if(menulevel == 24){
		if(button == up){
			dispoffneg = 1;
		}
		if (button == down){
			dispoffneg = 0;
		}
		if(button == enter){
			menuselect = menulevel % 10;
			menulevel /= 10;
			eeprom_write_byte((uint8_t*)dispoffnegaddr, dispoffneg);
		}
		return;
	}
	if(menulevel == 25){
		if(button == up){
			disponneg = 1;
		}
		if (button == down){
			disponneg = 0;
		}
		if(button == enter){
			menuselect = menulevel % 10;
			menulevel /= 10;
			eeprom_write_byte((uint8_t*)disponnegaddr, disponneg);
		}
		return;
	}

	if((button == down) && (menulevel == 0) && (menuselect < 4)){
			menuselect++;
		return;
	}
	if((button == down) && (menulevel == 1) && (menuselect < 5)){
		menuselect++;
		return;
	}
	if((button == down) && (menulevel == 2) && (menuselect < 5)){
		menuselect++;
		return;
	}
	if((button == down) && (menulevel == 3) && (menuselect < 2)){
		menuselect++;
		return;
	}

	if(button == enter){
		if((menulevel == 0) && (dispmode == normal)){
			dispmode = menu;
			menuselect = 1;
			return;
		}
		menulevel *= 10;
		menulevel += menuselect;
		menuselect = 1;
		return;
	}

}

void uart_sendbyte(char intface, char byte){
		while(!(UCSR0A & _BV(UDRE0)));
		intface = byte;
}

void uart_sendstring(char *str) 
{
	int i;
	for(i = 0; str[i] != 0; i++) {
		while(!(UCSR0A & _BV(UDRE0)));
		UDR0 = str[i];
	}
}


void requestdata(unsigned char datapoint, unsigned char itemno){
char byte;
unsigned char posx = 0;
	uart_sendbyte(UDR1, datapoint);
	while(1){
		while(!UCSR1A & 0b1000000);
	byte = UDR1;
	switch(byte){
		case 4: return;
		default: item[itemno][posx++] = byte; break;
		}
	}
}


ISR(USART0_RX_vect)
{
UCSR0B &= 0b01111111;	/* disables RXC interrupt for receive process */
UCSR0B |= 0b10000000;	/* enables RXC interrupt after receive complete */
}



ISR(TIMER2_OVF_vect)
{
	if(!sercount){	
		cli();
		LCD_update();
		//requestdata(0xf1, 0);
		sercount = 14;
		sei();
			
	}
	sercount --;
}

int main(void)
{
	setup();
	LCD_init(contrast, 0);
	LCD_update();
	//while(1);
	sercount = 127;
	sei();
	while(1){				
		if(!down_pin){
			buttonmanager(down);
			LCD_update();
			while(!down_pin);
		}
		if(!up_pin){
			buttonmanager(up);
			LCD_update();
			while(!up_pin);
		}
		if(!enter_pin){
			buttonmanager(enter);
			LCD_update();
			while(!enter_pin);
		}
		if(!escape_pin){
			buttonmanager(escape);
			LCD_update();
			while(!escape_pin);
		}
	}                
}
